(*
Copyright (C) 1999 Grischa Brockhaus <skullsoft@grischa.de>
modified      1999 Andre Grueneberg <andre@grueneberg.de>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*)

Unit Share;
(****************************************************************************)
(*                                                                          *)
(*   Unit fr erweiterte FileModes                                          *)
(*                                                                          *)
(*   Autor    : Grischa Brockhaus, SkullSoft                                *)
(*   erstellt : 01.07.93                                                    *)
(*   erweitert: 01.12.93                                                    *)
(*   Frei benutzbar                                                         *)
(****************************************************************************)

INTERFACE

Const

  (* Normale File Modes *)
  OnlyRead   = 0;
  OnlyWrite  = 1;
  ReadWrite  = 2;

  (* Erweiterte Filemodes fr geladendes SHARE *)
  DenyAll    = 16;
  DenyWrite  = 32;
  DenyRead   = 48;
  DenyNone   = 64;

Procedure SetFMode( Mode : Byte);
(* Setzen eines neuen FileModes, alter wird abgespeichert und kann mit
   RestoreFMode wieder hergestellt werden *)

Procedure RestoreFMode;
(* Gespeicherten FileMode wieder herstellen *)

Procedure OldFMode;
(* Den Turbo Pascal File Mode wieder herstellen *)

VAR
  ShareInstalled:Boolean; { TRUE = SHARE ist installiert }

(* Record Locking in files *)
PROCEDURE LockRecord(VAR Datei:File; Pos,Len:Longint; Var Ok :Boolean);
PROCEDURE UnLockRecord(VAR Datei:File; Pos,Len:Longint; Var Ok :Boolean);

{---------------------------------------------------------------------------}

IMPLEMENTATION
uses {$IfDef DPMI}
       MinDpmi,
     {$EndIf}
     {$ifdef VirtualPascal}
       Use32,
     {$endif}
     {$IfDef OS2}
       Os2base,
     {$EndIf}

     {$Ifdef Windows}
     Windos;
     {$Else}
     Dos;
     {$Endif}

var SaveFileMode : Byte;

{---------------------------------------------------------------------------}
Procedure SetFMode( Mode : Byte);
{ Andre hatte hier eine Asm Routine mit Byte Pointer auf Filemode, die nicht
  funkte.. }
begin
     SaveFileMode := FileMode;
     If ShareInstalled then FileMode := Mode or DenyNone
     else FileMode := Mode;
end;

{---------------------------------------------------------------------------}
Procedure RestoreFMode; Assembler;
{$IFDEF VIRTUALPASCAL}
{$Frame-}
{$ENDIF VIRTUALPASCAL}
asm
  mov al,SaveFileMode
  mov byte ptr FileMode,al
end;
{---------------------------------------------------------------------------}
Procedure OldFMode; Assembler;
{$IFDEF VIRTUALPASCAL}
{$Frame-}
{$ENDIF VIRTUALPASCAL}
asm
  mov al,ReadWrite
  mov ah,ShareInstalled
  jz @No_Share_Loaded
  or al,DenyNone
  @No_Share_Loaded:
  mov byte ptr FileMode,al
end;

{---------------------------------------------------------------------------}
{$IfDef Os2}
PROCEDURE LockRecord(VAR Datei:File; Pos,Len:Longint; Var Ok :Boolean);
var
   lock, unlock: FileLock;
begin
   fillchar(unlock,sizeof(FileLock),0);
   lock.lOffset := Pos;
   lock.lRange := Len;
   DosSetFileLocks(FileRec(Datei).handle,unlock,lock,5,1);
end;

PROCEDURE UnLockRecord(VAR Datei:File; Pos,Len:Longint; Var Ok :Boolean);
var
   lock, unlock: FileLock;
begin
   fillchar(lock,sizeof(FileLock),0);
   unlock.lOffset := Pos;
   unlock.lRange := Len;
   DosSetFileLocks(FileRec(Datei).handle,unlock,lock,5,1);
end;

begin
     ShareInstalled := True;
{$Else}
{---------------------------------------------------------------------------}
PROCEDURE LockRecord(VAR Datei:File; Pos,Len:Longint; Var Ok :Boolean);
{$ifdef win32}
begin
     Ok := false;
end;
{$else}
VAR
  Handle:word;
BEGIN
    Ok:=ShareInstalled;
    If not Ok then Exit;
    Handle:=FileRec(Datei).Handle;
    ASM
      mov ax,5C00h
      mov bx,Handle
      mov dx,word ptr Pos
      mov cx,word ptr pos[2]
      mov di,word ptr len
      mov si,word ptr len[2];
      mov Dos.DosError,0
      int 21h
      jnc @@Ok
      mov Dos.DosError, ax
      @@Ok:
    END;
    Ok:=DosError=0;
END;
{$endif win32}
{---------------------------------------------------------------------------}
PROCEDURE UnLockRecord(VAR Datei:File; Pos,Len:Longint; Var Ok :Boolean);
{$ifdef win32}
begin
     Ok := false;
end;
{$else}
VAR Handle:word;
BEGIN
    Ok:=ShareInstalled;
    If not Ok then Exit;
    Handle:=FileRec(Datei).Handle;
    ASM
      mov ax,5C01h
      mov bx,Handle
      mov dx,word ptr pos
      mov cx,word ptr pos[2]
      mov di,word ptr len
      mov si,word ptr len[2];
      mov Dos.DosError,0
      int 21h
      jnc @@Ok
      mov Dos.DosError, ax
      @@Ok:
    END;
    Ok:=DosError=0;
END;
{$endif win32}
{---------------------------------------------------------------------------}

{$IFDEF DPMI}
var Reg : TRealModeRegs;
{$ENDIF}
begin
     SaveFileMode:=FileMode;
     {$IFDEF DPMI}
     ClearRegs(Reg);
     Reg.Ax := $01000;
     RealModeInt($2F,Reg);
     ShareInstalled := Reg.AL = $0FF;
     {$ELSE}
     ASM
        MOV ShareInstalled, 00h
        MOV AX,01000h
        INT 2Fh
        CMP AL,0FFh
        JNE @1
        inc ShareInstalled
        @1:
     END;
     {$ENDIF DPMI}
     IF ShareInstalled THEN
        FileMode:=DenyNone OR Filemode;
{$EndIf OS2}
end.
